#coding:utf-8

'''
M500车机模拟服务类
'''
import binascii
import json
import os
import threading
import time
import traceback
from time import sleep

from lib.protocol.report.EventReport_protocol import EventReport_protocol
from lib.protocol.report.GPSReport_protocol import GPSReport_protocol
from lib.protocol.report.OBDReport_CAN_protocol import OBDReport_CAN_protocol
from lib.protocol.report.response.Common_response import Common_response
from lib.protocol.report.response.Update_response import Update_response
from lib.socket.service.ProtocolSimulaterDataService import ProtocolSimulaterDataService
from lib.socket.service.websocket_service import Websocket_service


class ProtocolSimulaterService():
    def __init__(self):
        self.data = {}                       #用来接收模拟器传过来的参数
        self.carData = {}                    #保存车辆行驶数据
        self.carDataObj = None               #管理车辆行驶数据对象
        self.socket = None
        self.sendDur = 5                     #设置默认多久发一条消息
        self.serviceStatus = 0               #服务状态，0表示未启动，1表示启动
        self.websocket = None                #网页与服务的通信socket
        self.timeout = 1                     #socket的超时时间
        self.gpsLine = []                    #GPS 轨迹
        self.gpsLineIndex = 0                #GPS 轨迹索引
        self.travelStatus = 0                #0，表示为行驶，1表示开始行驶
        self.carId = ""                      #车机号
        self.sn = 0                          #消息流水号
        self.travelDirection = 0             #行驶方向，0表示正向行驶，1表示反向行驶
        # 定义要发送的obd数据
        self.OBDdata = {"fireStatus":1,"ACCStatus":0,"engineSpeed":300,"speed":0,"meterMileage":6000,"totailMileage":600,"totalOilExpen":30,"totalRunTime":10}
        # 定义初始的obd数据，与上面的OBD数据保持一致，主要用于汽车行驶过程中数据变化量的计算
        self.OBDdataOri = {"fireStatus": 1, "ACCStatus": 0, "engineSpeed": 300, "speed": 0, "meterMileage": 6000,"totailMileage": 600, "totalOilExpen": 30, "totalRunTime": 10}

    #设置套接字
    def setSocket(self,data):
        self.socket = data
    def setSendDur(self,data):
        self.sendDur = data
    def setTimeout(self,data):
        self.timeout = data
    def setCarId(self,data):
        self.carId = data
    def setData(self,data):
        self.data = data
    def getWebsocket(self):
        return self.websocket


    def sendMsg(self,msg):
        self.socket.setTimeOut(self.timeout)
        self.socket.send(msg)
    def revMsg(self):
        self.socket.setTimeOut(self.timeout)
        return self.socket.receive()
    #发送消息，可指定消息的描述类型
    def serviceSendMsg(self,msg,type):          #type字段目前废掉没有实际意义
        self.sendMsg(msg)
        type = self.getMsgFunId(msg)
        self.websocket.send(">>>>" + type + "：" + msg)

    def serviceSend(self):
        while self.serviceStatus == 1:
            gpsMsg = ""
            OBDMsg = ""
            if self.travelStatus == 0:
                latitude = self.gpsLine[self.gpsLineIndex]["lat"]
                longitude = self.gpsLine[self.gpsLineIndex]["lng"]
                gpsMsg = self.genGPSMsg(latitude,longitude)
            elif self.travelStatus == 1:
                if self.gpsLineIndex < len(self.gpsLine) and self.gpsLineIndex != -1:
                    OBDMsg = self.genOBDMsg(self.OBDdata["fireStatus"],self.OBDdata["ACCStatus"],self.OBDdata["engineSpeed"], \
                                            self.OBDdata["speed"],self.OBDdata["meterMileage"],self.OBDdata["totailMileage"], \
                                            self.OBDdata["totalOilExpen"],self.OBDdata["totalRunTime"])
                    self.OBDdata["engineSpeed"] = 3000
                    self.OBDdata["speed"] = int(self.data["travelData"]["carSpeed"])
                    self.OBDdata["meterMileage"] = self.OBDdata["meterMileage"] + int(self.sendDur * (self.OBDdata["speed"] * 1000 / 3600))
                    self.OBDdata["totailMileage"] = self.OBDdata["totailMileage"] + int(self.sendDur * (self.OBDdata["speed"] * 1000  / 3600))
                    oilExpend = int(self.data["travelData"]["oilExpend"])
                    self.OBDdata["totalOilExpen"] = self.OBDdata["totalOilExpen"] + int((self.sendDur * (self.OBDdata["speed"] * 1000 / 3600)) * (1000 / (oilExpend *1000)))
                    self.OBDdata["totalRunTime"] = self.OBDdata["totalRunTime"] + self.sendDur
                    latitude = self.gpsLine[self.gpsLineIndex]["lat"]
                    longitude = self.gpsLine[self.gpsLineIndex]["lng"]
                    gpsMsg = self.genGPSMsg(latitude, longitude)
                    if self.travelDirection == 0:
                        self.gpsLineIndex = self.gpsLineIndex + 1     #正向行驶
                    else:
                        self.gpsLineIndex = self.gpsLineIndex - 1     #反向行驶
                elif self.gpsLineIndex == len(self.gpsLine) or self.gpsLineIndex == -1:
                    if int(self.data["travelData"]["travelLoop"]) == 0:
                        self.stopTravel()
                        self.websocket.send("gps轨迹跑完，自动停止行驶！")
                    else:
                        if self.travelDirection == 0:
                            self.gpsLineIndex = self.gpsLineIndex - 1
                            self.travelDirection = 1
                            self.websocket.send("gps轨迹正向行驶跑完，变换行驶方向......")
                        else:
                            self.gpsLineIndex = self.gpsLineIndex + 1
                            self.travelDirection = 0
                            self.websocket.send("gps轨迹反向行驶跑完，变换行驶方向......")
                        OBDMsg = self.genOBDMsg(self.OBDdata["fireStatus"], self.OBDdata["ACCStatus"],self.OBDdata["engineSpeed"], \
                                                self.OBDdata["speed"], self.OBDdata["meterMileage"],self.OBDdata["totailMileage"], \
                                                self.OBDdata["totalOilExpen"], self.OBDdata["totalRunTime"])
                        self.OBDdata["engineSpeed"] = 3000
                        self.OBDdata["speed"] = int(self.data["travelData"]["carSpeed"])
                        self.OBDdata["meterMileage"] = self.OBDdata["meterMileage"] + int(
                            self.sendDur * (self.OBDdata["speed"] * 1000 / 3600))
                        self.OBDdata["totailMileage"] = self.OBDdata["totailMileage"] + int(
                            self.sendDur * (self.OBDdata["speed"] * 1000 / 3600))
                        oilExpend = int(self.data["travelData"]["oilExpend"])
                        self.OBDdata["totalOilExpen"] = self.OBDdata["totalOilExpen"] + int(
                            (self.sendDur * (self.OBDdata["speed"] * 1000 / 3600)) * (1000 / (oilExpend * 1000)))
                        self.OBDdata["totalRunTime"] = self.OBDdata["totalRunTime"] + self.sendDur
                        latitude = self.gpsLine[self.gpsLineIndex]["lat"]
                        longitude = self.gpsLine[self.gpsLineIndex]["lng"]
                        gpsMsg = self.genGPSMsg(latitude, longitude)
                self.carDataObj.setTodayTotalMilleage(self.carData["curDayTravel"]["todayTotalMilleage"] + self.OBDdata["totailMileage"] - self.OBDdataOri["totailMileage"])
                self.carDataObj.setTheMilleage(self.carData["curDayTravel"]["theMilleage"] + self.OBDdata["totailMileage"] -self.OBDdataOri["totailMileage"])
                self.OBDdataOri["totailMileage"] = self.OBDdata["totailMileage"]
                self.carDataObj.setTodayTodayTotalOil(self.carData["curDayTravel"]["todayTotalOil"] + self.OBDdata["totalOilExpen"] - self.OBDdataOri["totalOilExpen"])
                self.carDataObj.setTheOil(self.carData["curDayTravel"]["theOil"] + self.OBDdata["totalOilExpen"] - self.OBDdataOri["totalOilExpen"])
                self.OBDdataOri["totalOilExpen"] = self.OBDdata["totalOilExpen"]
                self.carDataObj.setTodayTodayTotalTime(self.carData["curDayTravel"]["todayTotalTime"] + self.OBDdata["totalRunTime"] - self.OBDdataOri["totalRunTime"])
                self.carDataObj.setTheTime(self.carData["curDayTravel"]["theTime"] + self.OBDdata["totalRunTime"] - self.OBDdataOri["totalRunTime"])
                self.OBDdataOri["totalRunTime"] = self.OBDdata["totalRunTime"]
            if OBDMsg != "":
                self.sendMsg(OBDMsg)
                self.sn = self.sn + 1
                type = self.getMsgFunId(OBDMsg)
                info = type + ">>>>：" + OBDMsg
                self.websocket.send(info)
            if gpsMsg != "":
                sleep(0.1)
                self.sendMsg(gpsMsg)
                self.sn = self.sn + 1
                type = self.getMsgFunId(gpsMsg)
                info = type + ">>>>：" + gpsMsg
                self.websocket.send(info)
            sleep(self.sendDur)

    def serviceRev(self):
        self.serviceStatus = 2              #2代表只启动了接收消息的进程，1代表了接收和发送都启动了
        while self.serviceStatus != 0:
            self.socket.setTimeOut(self.timeout)
            d = self.revMsg()
            d = str(binascii.b2a_hex(d))[2:][:-1]
            type = self.getMsgFunId(d)
            info = type + "<<<<：" + d
            self.websocket.send(info)
            self.doResponse(d)

    #启动与页面交互的websockt服务
    def websocketService(self):
        self.websocket = Websocket_service()
        self.websocket.setHost("0.0.0.0")
        self.websocket.setPort(5005)
        self.websocket.startWebsocketServer()

    #启动定时发送消息和接收消息的服务
    def startService(self):
        self.serviceStatus = 1
        t1 = threading.Thread(target=self.serviceSend,args=())
        t1.start()

    #启动与页面交互的websocket服务
    def startWebsocketService(self):
        if self.websocket == None:
            t = threading.Thread(target=self.websocketService, args=())
            t.start()
            t2 = threading.Thread(target=self.serviceRev, args=())
            t2.start()
    #停止定时发送消息的服务
    def stopService(self):
        self.serviceStatus = 0
        self.gpsLine = []
        self.gpsLineIndex = 0
        self.travelStatus = 0

    def closeSocket(self):
        try:
            self.socket.close()
        except BaseException as e:
            # 打印异常信息
            traceback.print_exc()

    #停止websocket服务
    def stopWebsocketService(self):
        try:
            self.websocket.close()
            self.websocket = None
        except BaseException as e:
            # 打印异常信息
            traceback.print_exc()

    ########################################################
    # 开始行驶
    ########################################################
    def startTravel(self):
        self.travelStatus = 1
    ########################################################
    # 停止行驶
    ########################################################
    def stopTravel(self):
        self.travelStatus = 0
        self.serviceStatus = 0

    #获取收到消息的功能id
    def getMsgFunId(self,msg):
        funId = msg[26:30]
        return funId
    #收到 某些类型的消息后执行回复操作
    def doResponse(self,msg):
        msgFunId = self.getMsgFunId(msg)
        if msgFunId == "8205":
            msg = Common_response(resId="8205").generateCommonMsg()
            self.sendMsg(msg)
            self.sn = self.sn + 1
            type = self.getMsgFunId(msg)
            self.websocket.send(type + ">>>>设置GPSR通信参数应答：" + msg)
        elif msgFunId == "8206":
            msg = Common_response(resId="8206").generateCommonMsg()
            self.sendMsg(msg)
            self.sn = self.sn + 1
            type = self.getMsgFunId(msg)
            self.websocket.send(type + ">>>>设置车辆OBD适配信息应答：" + msg)
        elif msgFunId == "8300":
            msg = Update_response().generateUpdateMsg()
            self.sendMsg(msg)
            self.sn = self.sn + 1
            type = self.getMsgFunId(msg)
            self.websocket.send(type + ">>>>升级_平台通知终端远程升级应答：" + msg)

    #设置GPS轨迹
    def setGpsLine(self,fileName):
        with open("data/protocolTools/GPSLines/" + fileName,"r",encoding="utf-8") as fi:
            content = fi.read()
            conJson = json.loads(content)
            self.gpsLine = conJson["GPSLine"]

    #点火，发送点火事件
    def fireOn(self):
        if not os.path.exists("data/protocolTools/carData/" + self.carId + ".json"):
            psdsObj = ProtocolSimulaterDataService()
            data = psdsObj.genDataTemplate()
            psdsObj.writeToFile("data/protocolTools/carData/" + self.carId + ".json",data)
        #读取车机行驶数据
        with open("data/protocolTools/carData/" + self.carId + ".json", "r", encoding="utf-8") as fi:
            content = fi.read()
            conJson = json.loads(content)
            conJson["curDayTravel"]["theMilleage"] = 0             # 今日行驶总里程
            conJson["curDayTravel"]["theOil"] = 0                  # 今日行驶总油耗
            conJson["curDayTravel"]["theTime"] = 0                 # 今日行驶总时间
            timeStamp = time.time()
            timeArray = time.localtime(timeStamp)
            dateTimeM = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
            dateM = time.strftime("%Y-%m-%d", timeArray)
            timeM = time.strftime("%H:%M:%S", timeArray)
            dataFile = self.carId + ".json"
            self.carDataObj = ProtocolSimulaterDataService("data/protocolTools/carData/", dataFile)
            self.carDataObj.setData(conJson)
            if dateM == conJson["time"]["date"]:
                pass
            else:                                                          #如果不是当天日期，则将日期设置为当天，并写入车辆数据文件
                conJson["time"]["dateTime"] = dateTimeM
                conJson["time"]["date"] = dateM
                conJson["time"]["time"] = timeM
                conJson["curDayTravel"]["todayTotalMilleage"] = 0             # 今日行驶总里程
                conJson["curDayTravel"]["todayTotalOil"] = 0                  # 今日行驶总油耗
                conJson["curDayTravel"]["todayTotalTime"] = 0                 # 今日行驶总时间
                self.carDataObj.setTodayTotalMilleage(0)
                self.carDataObj.setTodayTodayTotalOil(0)
                self.carDataObj.setTodayTodayTotalTime(0)
                self.carDataObj.setDateTime2file(dateTimeM)
                self.carDataObj.setData2file(dateM)
                self.carDataObj.setTime2file(timeM)
            self.carData = conJson
        fireOnEventObj = EventReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsData = self.genGPSData(self.gpsLine[0]["lat"], self.gpsLine[0]["lng"])
        fireOnEventObj.setGPSPkg(gpsData)
        fireOnEventObj.setEventType("0010")
        firOnEventMsg = fireOnEventObj.generateEventMsg()
        type = self.getMsgFunId(firOnEventMsg)
        self.sendMsg(firOnEventMsg)
        self.sn = self.sn + 1
        self.websocket.send(type + ">>>>：" + firOnEventMsg)
        sleep(0.1)
        gpsMsg = self.genGPSMsg(self.gpsLine[0]["lat"],self.gpsLine[0]["lng"])
        type = self.getMsgFunId(gpsMsg)
        self.sendMsg(gpsMsg)
        self.sn = self.sn + 1
        self.websocket.send(type + ">>>>：" + gpsMsg)
        sleep(0.1)
        OBDMsg = self.genOBDMsg()
        type = self.getMsgFunId(OBDMsg)
        self.sendMsg(OBDMsg)
        self.sn = self.sn + 1
        self.websocket.send(type + ">>>>：" + OBDMsg)

    # 熄火，发送熄火事件
    def fireOff(self):
        gpsLineIndex = self.gpsLineIndex
        if gpsLineIndex >= len(self.gpsLine):
            gpsLineIndex = gpsLineIndex - 1
        fireOffEventObj = EventReport_protocol(DEV_ID=self.carId)
        gpsData = self.genGPSData(self.gpsLine[gpsLineIndex]["lat"],self.gpsLine[gpsLineIndex]["lng"])
        fireOffEventObj.setGPSPkg(gpsData)
        fireOffEventObj.setEventType("0011")
        fireOffEventMsg = fireOffEventObj.generateEventMsg()
        type = self.getMsgFunId(fireOffEventMsg)
        self.sendMsg(fireOffEventMsg)
        self.websocket.send(type + ">>>>：" + fireOffEventMsg)
        sleep(0.1)
        gpsMsg = self.genGPSMsg(self.gpsLine[gpsLineIndex]["lat"], self.gpsLine[gpsLineIndex]["lng"])
        type = self.getMsgFunId(gpsMsg)
        self.sendMsg(gpsMsg)
        self.sn = self.sn + 1
        self.websocket.send(type + ">>>>：" + gpsMsg)

    #根据特定参数，生成GPS消息
    def genGPSMsg(self,latitude,longtitude):
        gpsObj = GPSReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsObj.setLatitude(latitude)
        gpsObj.setLongitude(longtitude)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        gpsObj.setUTCTime(UTCTime)
        gpsObj.setGPSTimestamp(timeS)
        msg = gpsObj.generateGpsMsg()
        return msg
    #根据特定参数，生成GPS消息体，不包含消息头
    def genGPSData(self,latitude,longtitude):
        gpsObj = GPSReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsObj.setLatitude(latitude)
        gpsObj.setLongitude(longtitude)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        gpsObj.setUTCTime(UTCTime)
        gpsObj.setGPSTimestamp(timeS)
        data = gpsObj.generateGpsData()
        return data

    # 根据特定参数，生成OBD CAN消息
    def genOBDMsg(self,fireStatus=1,ACCStatus=0,engineSpeed=300,speed=0,meterMileage=6000, \
                  totailMileage=600,totalOilExpend=30,totalRunTime=10):
        OBDObj = OBDReport_CAN_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        OBDObj.setInfoTime(UTCTime)
        OBDObj.setFireStatus(fireStatus)
        OBDObj.setACCStatus(ACCStatus)
        OBDObj.setEngineSpeed(engineSpeed)                 # 设置发动机转速
        OBDObj.setSpeed(speed)                             # 设置车辆速度
        OBDObj.setMeterMileage(meterMileage)               # 设置仪表里程值
        OBDObj.setTotalMileage(totailMileage)              # 设置总里程值
        OBDObj.setTotalOilExpend(totalOilExpend)           # 设置总耗油量
        OBDObj.setTotalRunTime(totalRunTime)               # 设置车辆运行时间
        msg = OBDObj.generateOBDReportCANMsg()
        return msg











